@@ -17,7 +17,7 @@ namespace ioh
         class CpuTimer
         {
             /**
-             * \brief A info message to be printed to stdout when the timer completes
+             * \brief A info message to be logged to stdout when the timer completes
              */
             std::string info_msg_;
             /**
@@ -42,7 +42,8 @@ namespace ioh
                 IOH_DBG(progress,
                         fmt::format(
                             "{}CPU Time: {:d} ms", info_msg_,
-                            std::chrono::duration_cast<std::chrono::microseconds>(Clock::now() - start_time_).count()));
+                            std::chrono::duration_cast<std::chrono::microseconds>(Clock::now() - start_time_).count())
+                    );
             }
         };
 
