@@ -4,6 +4,7 @@
 #include <map>
 #include <string>
 #include <numeric>
+#include <memory>
 
 #include "loggers.hpp"
 
@@ -49,6 +50,12 @@ namespace logger {
 
             /** Constructor with time, then quality. */
             Point(size_t nb_evals, double quality) : qual(quality), time(nb_evals) { }
+
+            friend std::ostream& operator<< (std::ostream& out, const Point& p )
+            {
+                out << "{\"qual\":" << p.qual << ",\"time\":" << p.time << "}";
+                return out;
+            }
         };
 
         /** A point and its corresponding run index.
@@ -69,6 +76,14 @@ namespace logger {
 
             /** Constructor with time, then quality. */
             RunPoint(size_t t, double q, size_t r) : Point(q,t), run(r) { }
+
+            friend std::ostream& operator<< (std::ostream& out, const RunPoint& p )
+            {
+                out << "{\"run\":" << p.run << ", \"point\":";
+                out << static_cast<const Point&>(p);
+                out << "}";
+                return out;
+            }
         };
 
         /** A set of non-dominated points. */
@@ -194,28 +209,36 @@ namespace logger {
         };
 
         /** Return the front at the given cursor. */
-        [[nodiscard]] const eaf::Front& data(const Cursor cur) const
+        [[nodiscard]] const eaf::Front& data(const Cursor& cur) const
         {
 #ifndef NDEBUG
             assert(_data.count(cur.suite) > 0);
-            assert(   _data.at(cur.suite).count(cur.pb) > 0);
-            assert(   _data.at(cur.suite)   .at(cur.pb).count(cur.dim) > 0);
-            assert(   _data.at(cur.suite)   .at(cur.pb)   .at(cur.dim).count(cur.ins) > 0);
-            assert(   _data.at(cur.suite)   .at(cur.pb)   .at(cur.dim)   .at(cur.ins).count(cur.run) > 0);
+            assert(_data.at(cur.suite).count(cur.pb) > 0);
+            assert(_data.at(cur.suite).at(cur.pb).count(cur.dim) > 0);
+            assert(_data.at(cur.suite).at(cur.pb).at(cur.dim).count(cur.ins) > 0);
+            assert(_data.at(cur.suite).at(cur.pb).at(cur.dim).at(cur.ins).count(cur.run) > 0);
 #endif
             return _data.at(cur.suite).at(cur.pb).at(cur.dim).at(cur.ins).at(cur.run);
 
         }
 
     public:
+        /** Constructor.
+         * 
+         * @warning A classical error in C++ is to instantiate a constructor with empty parentheses.
+         *          Here, you should take care of instantiating without any parenthese at all:
+         *          @code
+         *              ioh::logger::EAF my_logger;
+         *          @endcode
+         */
         EAF() : Logger(), _has_problem_type(false)
         {
             // Do not use the Logger's constructor, to avoid passing references to uninitialized members.
             // /!\ needed by the algorithm, do not change unless you know what you're doing.
             triggers_.insert(std::ref(_on_improvement));
             // /!\ needed by the related eaf::stat::* classes.
-            properties_.insert_or_assign(_transformed_y_best.name(), _transformed_y_best);
-            properties_.insert_or_assign(       _evaluations.name(), _evaluations);
+            properties_.insert_or_assign(_y_best.name(), _y_best);
+            properties_.insert_or_assign(_evaluations.name(), _evaluations);
         }
 
         /** Set the current suite name.
@@ -242,15 +265,16 @@ namespace logger {
 
             IOH_DBG(note, "Attach to: pb=" << _current.pb << ", dim=" << _current.dim << ", ins=" << _current.ins << ", run=" << _current.run)
 #ifndef NDEBUG
-            if(_has_problem_type and _current_problem_type != problem.optimization_type) {
+            if(_has_problem_type and _current_problem_type.type() != problem.optimization_type.type()) {
                 IOH_DBG(warning, "different types of problems are mixed, you will not be able to compute levels")
             }
 #endif
             _current_problem_type = problem.optimization_type;
             _has_problem_type = true;
 #ifndef NDEBUG
-            if(problem.optimization_type == common::OptimizationType::Minimization) {
-                _current_best =  std::numeric_limits<double>::infinity();
+            // Reset _current_best (double check of trigger::on_improvement behaviour).
+            if(_current_problem_type == common::OptimizationType::MIN) {
+                _current_best = std::numeric_limits<double>::infinity();
             } else {
                 _current_best = -std::numeric_limits<double>::infinity();
             }
@@ -262,19 +286,23 @@ namespace logger {
         {
             IOH_DBG(debug, "EAF called after improvement")
             // Access the properties that were instantiated in the constructor.
-            const std::optional<double> transformed_y_best = properties_.at("transformed_y_best").get()(log_info);
-            const std::optional<double> evaluations        = properties_.at("evaluations").get()(log_info);
+            const std::optional<double> y_best      = properties_.at(_y_best.name()).get()(log_info);
+            const std::optional<double> evaluations = properties_.at(_evaluations.name()).get()(log_info);
 #ifndef NDEBUG
-            assert(transformed_y_best); // Assert that the optional holds a value, which should be the case here.
+            assert(y_best); // Assert that the optional holds a value, which should be the case here.
             assert(evaluations);
 
             // If trigger::on_improvement does its job, we always have an improvement here.
-            assert(common::compare_objectives(transformed_y_best.value(),_current_best,_current_problem_type));
-            _current_best = transformed_y_best.value();
+            IOH_DBG(debug, "y_best=" << y_best.value()
+                <<  (_current_problem_type == common::OptimizationType::MIN ? " <" : " >")
+                << " current_best=" << _current_best << " ?");
+            
+            assert(_current_problem_type(y_best.value(), _current_best));
+            _current_best = y_best.value();
 #endif
             eaf::Front& f = current_front();
             f.push_back( eaf::RunPoint(
-                    transformed_y_best.value(),
+                    y_best.value(),
                     static_cast<size_t>(evaluations.value()),
                     _current.run
                 ));
@@ -283,24 +311,31 @@ namespace logger {
         /** Reset the state. */
         void reset() override
         {
+            IOH_DBG(debug, "EAF reset")
             Logger::reset();
-            IOH_DBG(note, "reset")
+#ifndef NDEBUG
+            if(_current_problem_type == common::OptimizationType::MIN) {
+                _current_best =  std::numeric_limits<double>::infinity();
+            } else {
+                _current_best = -std::numeric_limits<double>::infinity();
+            }
+#endif
         }
 
         common::OptimizationType optimization_type() const
         {
             assert(_has_problem_type);
-            return _current_problem_type;
+            return _current_problem_type.type();
         }
 
     protected:
 #ifndef NDEBUG
-        // Use to double check that trigger::on_improvement do its job.
+        // Use to double check that trigger::on_improvement does its job.
         // It should not be necessary otherwise.
         double _current_best;
 #endif
         bool _has_problem_type;
-        common::OptimizationType _current_problem_type;
+        common::FOptimizationType _current_problem_type;
 
         /** The main data structure in which log events are stored. */
         Suites _data;
@@ -325,7 +360,7 @@ namespace logger {
         watch::Evaluations _evaluations;
 
         //! Property watching the objective function value.
-        watch::TransformedYBest _transformed_y_best;
+        watch::CurrentBestY _y_best;
 
 
     }; // class EAF
@@ -352,7 +387,7 @@ namespace logger {
             logger::EAF logger;
             suite.attach_logger(logger);
             // [call the problem... the logger will store the quality/time Pareto fronts for each run.
-            logger::eaf::stat::Levels levels_at(common::OptimizationType::Minimization, {0, nb_runs/2, nb_runs-1}); // Will computes 3 level sets.
+            logger::eaf::stat::Levels levels_at(common::OptimizationType::MIN, {0, nb_runs/2, nb_runs-1}); // Will computes 3 level sets.
             auto levels = levels_at(logger); // Actually compute the levels.
          * @endcode
          *
@@ -409,7 +444,7 @@ namespace logger {
                 std::vector<size_t> _attlevels;
 
                 /** Whether we minimize or maximize. */
-                const common::OptimizationType _optim_type;
+                const common::FOptimizationType _f_optim;
 
                 /** Compare two objective function values.
                  * 
@@ -421,7 +456,7 @@ namespace logger {
                  */
                 bool is_better(const double q1, const double q2)
                 {
-                    return common::compare_objectives(q1, q2, _optim_type);
+                    return _f_optim(q1, q2);
                 }
                 
                 /** Compare two objective function values.
@@ -455,7 +490,7 @@ namespace logger {
                  */
                 Levels(const common::OptimizationType optim_type, std::vector<size_t> attainment_levels = {})
                 : _attlevels(attainment_levels)
-                , _optim_type(optim_type)
+                , _f_optim{optim_type}
                 { }
 
                 /** Extract the runs from the logger and computes the attainment levelsets. */
@@ -465,7 +500,7 @@ namespace logger {
                     // But, there is no way to ensure that all problems seen by the loggers
                     // have been of the same type.
                     // This is at least a check.
-                    assert(_optim_type == logger.problem().optimization_type);
+                    assert(_f_optim == logger.problem().value().optimization_type.type());
                     
                     // Input:
                     std::vector<eaf::Front> fronts;
@@ -502,7 +537,7 @@ namespace logger {
                     // Sort the two sets of data.
                     std::sort(std::begin(data_time),std::end(data_time),  ascending_time); // Time always ascend.
 
-                    if(_optim_type == common::OptimizationType::Minimization ) {
+                    if(_f_optim == common::OptimizationType::MIN ) {
                         std::sort(std::begin(data_qual),std::end(data_qual), descending_qual); 
                     } else {
                         std::sort(std::begin(data_qual),std::end(data_qual),  ascending_qual); 
@@ -587,7 +622,7 @@ namespace logger {
                                 // assert(nb_attained < level);
                                 assert(it < total_nb_points);
                                 assert(iq-1 < total_nb_points);
-                                IOH_DBG(debug, ">> Front level point: (" << data_time[it].time << "," << data_qual[iq-1].qual << ")")
+                                IOH_DBG(debug, ">> Front level point: (time=" << data_time[it].time << ",qual=" << data_qual[iq-1].qual << ")")
                                 level_front.push_back( eaf::RunPoint(
                                         data_time[it].time,
                                         data_qual[iq-1].qual,
@@ -602,7 +637,7 @@ namespace logger {
                         assert(level_front.size() > 0);
                         assert(levels.find(level) == std::end(levels));
                         levels[level] = level_front;
-                        #ifndef NDEBUG
+#ifndef NDEBUG
                             IOH_DBG(xdebug,">> Involved runs:")
                             IOH_DBG(xdebug,">>> " << level_front[0].run)
                             for(size_t i = 1; i < level_front.size(); ++i) {
@@ -611,7 +646,7 @@ namespace logger {
                                 }
                                 assert(level_front[i].run == level_front[i-1].run);
                             }
-                        #endif
+#endif
                     } // l in levels
 
                     IOH_DBG(note, "Ended with " << levels.size() << " levels")
@@ -624,7 +659,6 @@ namespace logger {
 
         /** Computes attainment levels of the Empirical Attainment Function.
          * 
-         * @param optim_type Whether we target a minimization or a maximization problem.
          * @param logger the logger holding the data.
          * @param levels The desired level indices (empty=all, the default).
          * @returns A map associating a level index to the corresponding set of non-dominated quality/time points.
@@ -633,9 +667,9 @@ namespace logger {
          * 
          * @ingroup EAF
          */
-        inline Levels::Type levels(const common::OptimizationType optim_type, const EAF& logger, std::vector<size_t> levels = {} )
+        inline Levels::Type levels(const EAF& logger, std::vector<size_t> levels = {} )
         {
-            Levels levels_of(optim_type, levels);
+            Levels levels_of(logger.optimization_type(), levels);
             return levels_of(logger);
         }
 
@@ -661,6 +695,9 @@ namespace logger {
          * @note Letting the class computes its nadir point will be more expensive than
          *       using an extremum that you would know from bounds.
          * 
+         * @warning This should always compute a metric that should be maximized in order to improve the performance.
+         *          (i.e. *under* the curve for maximization problems, *above* the curve for minimization problems)
+         * 
          * @ingroup EAF_API
          */
         template<class T>
@@ -669,29 +706,48 @@ namespace logger {
             protected:
                 //! Initialization check flag.
                 bool _has_nadir;
+
                 //! The worst reference point.
                 eaf::Point _nadir;
+
                 //! The problem type (i.e. min or max).
-                const common::OptimizationType _optim_type;
+                const common::FOptimizationType _is_better;
+
+                //! Shift of the nadir point regarding the extremum of data.
+                static const size_t _shift_time = 1;
+                const double _shift_qual;
 
                 /** Computes the extremum point as the worst coordinates on both quality and time axis for all front points.
                  */
                 eaf::Point extremum(const eaf::Front& front) const
                 {
-                    size_t worst_time;
-                    double worst_qual;
-                    if(_optim_type == common::OptimizationType::Minimization) {
-                        worst_time =  std::numeric_limits<size_t>::lowest();
-                        worst_qual = -std::numeric_limits<double>::infinity();
+                    // size_t worst_time;
+                    // double worst_qual;
+                    // if(_is_better.type() == common::OptimizationType::MIN) {
+                    //     worst_time =  std::numeric_limits<size_t>::lowest();
+                    //     worst_qual = -std::numeric_limits<double>::infinity();
+                    // } else {
+                    //     worst_time =  std::numeric_limits<size_t>::infinity();
+                    //     worst_qual =  std::numeric_limits<double>::infinity();
+                    // }
+                    // for(const auto& p : front) {
+                    //     if(p.time > worst_time) { worst_time = p.time; }
+                    //     if(_is_better(worst_qual, p.qual)) { worst_qual = p.qual; }
+                    // }
+                    // return eaf::Point(worst_qual, worst_time);
+                    double nadir_qual;
+                    size_t nadir_time = std::numeric_limits<size_t>::lowest();
+
+                    if(_is_better.type() == common::OptimizationType::MIN) {
+                        nadir_qual = -std::numeric_limits<double>::infinity();
                     } else {
-                        worst_time =  std::numeric_limits<size_t>::infinity();
-                        worst_qual =  std::numeric_limits<double>::infinity();
+                        nadir_qual =  std::numeric_limits<double>::infinity();
                     }
-                    for(const auto& p : front) {
-                        if(p.time > worst_time) { worst_time = p.time; }
-                        if(common::compare_objectives(worst_qual, p.qual, _optim_type)) { worst_qual = p.qual; }
+                    for(const auto& candidate_nadir : front) {
+                        if(candidate_nadir.time > nadir_time) { nadir_time = candidate_nadir.time; }
+                        if(_is_better(nadir_qual, candidate_nadir.qual)) { nadir_qual = candidate_nadir.qual; }
                     }
-                    return eaf::Point(worst_qual, worst_time);
+                    return eaf::Point(nadir_qual, nadir_time);
                 }
 
                 /** Main interface.
@@ -705,30 +761,44 @@ namespace logger {
                 using Type = typename Stat<T>::Type;
                 
                 /** Constructor without a nadir reference point. */
-                NadirStat(const common::OptimizationType optim_type) : _has_nadir(false), _nadir(), _optim_type(optim_type) { }
+                NadirStat(const common::OptimizationType optim_type, const double shift_qual = 1e-6) : _has_nadir(false), _nadir(), _is_better{optim_type}, _shift_qual(shift_qual) {
+                    IOH_DBG(note, "NadirStat without a nadir");
+                }
 
-                /** Constructor with a nadir reference point. */
-                NadirStat(const common::OptimizationType optim_type, eaf::Point nadir) : _has_nadir(true), _nadir(nadir), _optim_type(optim_type) { }
+                /** Constructor with a nadir reference point.
+                 *
+                 * @warning Using this constructor, you are responsible of passing the correct nadir point.
+                 * 
+                 * @warning Note that you must shift it to ensure that the statistic is always non-zero.
+                 */
+                NadirStat(const common::OptimizationType optim_type, eaf::Point nadir) : _has_nadir(true), _nadir(nadir), _is_better{optim_type},
+                    _shift_qual(std::numeric_limits<double>::signaling_NaN()) { }
 
                 /** Update the nadir point (if not previously indicated) and then calls NadirStat::call. */
                 T operator()(const eaf::Levels::Type& levels) override
                 {
                     if(not _has_nadir) {
-                        size_t worst_time;
-                        double worst_qual;
-                        if(_optim_type == common::OptimizationType::Minimization) {
-                            worst_time =  std::numeric_limits<size_t>::lowest();
-                            worst_qual = -std::numeric_limits<double>::infinity();
+                        double nadir_qual;
+                        double epsil_qual;
+                        size_t nadir_time = 0;
+                        const size_t epsil_time = 1;
+
+                        if(_is_better.type() == common::OptimizationType::MIN) {
+                            nadir_qual = -std::numeric_limits<double>::infinity();
+                            // epsil_qual =  1;//std::numeric_limits<double>::epsilon();
+                            epsil_qual =  _shift_qual;
                         } else {
-                            worst_time =  std::numeric_limits<size_t>::infinity();
-                            worst_qual =  std::numeric_limits<double>::infinity();
+                            nadir_qual =  std::numeric_limits<double>::infinity();
+                            epsil_qual = -1;//std::numeric_limits<double>::epsilon();
+                            epsil_qual = -_shift_qual;
                         }
-                        for(const auto& [level,front] : levels) {
-                            eaf::Point nadir = extremum(front);
-                            if(nadir.time > worst_time) { worst_time = nadir.time; }
-                            if(common::compare_objectives(worst_qual, nadir.qual, _optim_type)) { worst_qual = nadir.qual; }
+                        for(const auto& [_,front] : levels) {
+                            eaf::Point candidate_nadir = extremum(front);
+                            if(candidate_nadir.time > nadir_time) { nadir_time = candidate_nadir.time; }
+                            if(_is_better(nadir_qual, candidate_nadir.qual)) { nadir_qual = candidate_nadir.qual; }
                         }
-                        _nadir = eaf::Point(worst_qual, worst_time);
+                        IOH_DBG(debug,"Estimated nadir: (time=" << nadir_time+epsil_time << ",qual=" << nadir_qual+epsil_qual << ")" );
+                        _nadir = eaf::Point(nadir_qual+epsil_qual, nadir_time+epsil_time);
                         _has_nadir = true;
                     }
 
@@ -745,9 +815,12 @@ namespace logger {
              * This computes the surface encompassed by each levelset, up to the extremum point,
              * and returns a map between the level index and of the corresponding surfaces.
              * 
+             * @warning This will always compute a metric that should be maximized in order to improve the performance.
+             *          (i.e. *under* the curve for maximization problems, *above* the curve for minimization problems)
+             * 
              * @ingroup EAF
              */
-            class Surface : public NadirStat<std::map<size_t,double>>
+            class Surfaces : public NadirStat<std::map<size_t,double>>
             {
                 protected:
                     /** The NadirStat interface. */
@@ -757,15 +830,17 @@ namespace logger {
                         for(auto [level,front] : levels) {
                             results[level] = surface(front);
                         }
+                        IOH_DBG(debug,"Surfaces of " << levels.size() << " levels");
+                        assert(results.size() == levels.size());
                         return results;
                     }
                     
                 public:
                     /** Constructor without a nadir reference point. */
-                    Surface(const common::OptimizationType optim_type) : NadirStat(optim_type) { }
+                    Surfaces(const common::OptimizationType optim_type) : NadirStat(optim_type) { }
 
                     /** Constructor with a nadir reference point. */
-                    Surface(const common::OptimizationType optim_type, eaf::Point nadir) : NadirStat(optim_type, nadir) { }
+                    Surfaces(const common::OptimizationType optim_type, eaf::Point nadir) : NadirStat(optim_type, nadir) { }
 
                     // This is public so as to allow for single loop computations in Volume::call.
                     /** Computes the surface of the given non-dominated set of points.
@@ -774,36 +849,47 @@ namespace logger {
                      */
                     double surface(eaf::Front front) const
                     {
+                        IOH_DBG(xdebug,"Front of size: " << front.size());
                         assert(_has_nadir);
                         double n_qual = _nadir.qual;
                         double surface = 0;
 
-                        std::sort(std::begin(front), std::end(front), eaf::descending_qual);
+                        if(this->_is_better.type() == common::OptimizationType::MIN) {
+                            std::sort(std::begin(front), std::end(front), eaf::descending_qual);
+                        } else {
+                            std::sort(std::begin(front), std::end(front), eaf::ascending_qual);
+                        }
+                        
                         for(const auto& p : front) {
                             surface +=   std::fabs(n_qual - p.qual) 
                                        * std::labs(static_cast<long>(_nadir.time) - static_cast<long>(p.time)); // cast because time are unsigned
                             n_qual = p.qual;
+                            assert(surface > 0);
                         }
+                        IOH_DBG(xdebug, "Surface of front = " << surface);
                         return surface;
                     }
             };
 
             /** Computes the "hypervolumes" of the given attainment levelsets.
              * 
-             * This computes the surface encompassed by each levelset, up to the extremum point,
+             * This computes the surfaces encompassed by each levelset, up to the extremum point,
              * and returns a map between the level index and of the corresponding surfaces.
              * 
              * @warning The nadir reference point is computed automatically as the extremum of all the levels.
              *          Thus, if you call this function several times on different levels,
              *          the surfaces will (may) be computed with different nadir points.
-             *          To indicate your own nadir point, use the stat::Surface class directly.
+             *          To indicate your own nadir point, use the stat::Surfaces class directly.
+             * 
+             * @warning This will always compute a metric that should be maximized in order to improve the performance.
+             *          (i.e. *under* the curve for maximization problems, *above* the curve for minimization problems)
              * 
              * @ingroup EAF
              */
-            inline Surface::Type surface(const common::OptimizationType optim_type, const eaf::Levels::Type& levels)
+            inline Surfaces::Type surfaces(const common::OptimizationType optim_type, const eaf::Levels::Type& levels)
             {
-                Surface surface_of(optim_type);
-                return surface_of(levels);
+                Surfaces surfaces_of(optim_type);
+                return surfaces_of(levels);
             }
                                   
             /** Computes the volume of the whole empirical attainement function.
@@ -813,6 +899,9 @@ namespace logger {
              * 
              * @note If only a subset of the levelsets are given, the true volume will be underestimated.
              * 
+             * @warning This will always compute a metric that should be maximized in order to improve the performance.
+             *          (i.e. *under* the curve for maximization problems, *above* the curve for minimization problems)
+             * 
              * @ingroup EAF
              */
             class Volume : public NadirStat<double>
@@ -821,15 +910,28 @@ namespace logger {
 
                     double call(const eaf::Levels::Type& levels) override
                     {
-                        Surface s(_optim_type, _nadir);
+                        IOH_DBG(debug, "Volume of " << levels.size() << " levels");
+                        Surfaces s(this->_is_better.type(), _nadir);
                         // Do not compute surfaces right here to avoid two loops on levels instead of one.
 
                         double volume = 0;
                         auto prev_level = -1;
                         for(const auto& [level,front] : levels) {
-                            volume += s.surface(front) * (static_cast<double>(level) - prev_level);
+                            const auto fsurf = s.surface(front);
+#ifndef NDEBUG
+                                if(fsurf == 0) {
+                                    IOH_DBG(warning, "Level " << level << " has a null surface.");
+                                }
+#endif
+                            volume += fsurf * (static_cast<double>(level) - prev_level);
                             prev_level = static_cast<int>(level);
                         }
+                        IOH_DBG(debug, "Volume = " << volume);
+#ifndef NDEBUG
+                            if(volume == 0) {
+                                IOH_DBG(warning, "Null volume, if some front have single points, set a nadir point.");
+                            }
+#endif
                         return volume;
                     }
                 public:
@@ -848,6 +950,9 @@ namespace logger {
              *          the surfaces will (may) be computed with different nadir points.
              *          To indicate your own nadir point, use the stat::Volume class directly.
              * 
+             * @warning This will always compute a metric that should be maximized in order to improve the performance.
+             *          (i.e. *under* the curve for maximization problems, *above* the curve for minimization problems)
+             * 
              * @ingroup EAF
              */
             inline double volume(const common::OptimizationType optim_type, const eaf::Levels::Type& levels)
@@ -861,6 +966,9 @@ namespace logger {
              * If you don't pass the number of runs, it will be guessed by looking at
              * the number of runs in the very first logs.
              * 
+             * @warning This will compute a metric that should be maximized in order to improve the performance.
+             *          (i.e. *under* the curve for maximization problems, *above* the curve for minimization problems)
+             * 
              * @ingroup EAF
              */
             inline double volume(const EAF& logger, double error_min, double error_max, size_t evals_min, size_t evals_max, size_t nb_runs = 0)
@@ -869,23 +977,61 @@ namespace logger {
                 assert(evals_max > evals_min);
 
                 // Prepare the instance with the Nadir point.
-                Volume volume_of(logger.optimization_type(), Point(error_max, evals_max));
+                std::unique_ptr<Volume> volume_of = nullptr;
+                if(logger.optimization_type() == common::OptimizationType::MIN) {
+                    volume_of = std::make_unique<Volume>(logger.optimization_type(), Point(error_max, evals_max));
+                } else {
+                    volume_of = std::make_unique<Volume>(logger.optimization_type(), Point(error_min, evals_max));
+                }
 
                 // Computes all levels.
-                auto v = volume_of(levels(logger.optimization_type(), logger, {}));
+                auto v = (*volume_of)(levels(logger, {}));
 
                 // Get the number of runs from the first instance.
                 // WARNING: this assume that all instances have the same number of runs,
                 // an assumption that is not enforced in the logger.
                 if(nb_runs == 0) {
-                    nb_runs = logger.data().at(0).at(0).at(0).at(0).size();
+                    const auto suite = logger.data().begin();
+                    assert(suite != logger.data().end());
+                    const auto pb = suite->second.begin();
+                    assert(pb != suite->second.end());
+                    const auto dim = pb->second.begin();
+                    assert(dim != pb->second.end());
+                    const auto ins = dim->second.begin();
+                    assert(ins != dim->second.end());
+                    nb_runs = ins->second.size();
 #ifndef NDEBUG
-                    // FIXME double check that all instances have that many runs
+                    // Double check that all instances have that many runs
+                    for(const auto& suite_pb : logger.data()) {
+                        for(const auto& pb_dim : suite_pb.second) {
+                            for(const auto& dim_ins : pb_dim.second) {
+                                for(const auto& ins_runs : dim_ins.second) {
+                                    for(const auto& run_front : ins_runs.second) {
+                                       assert(run_front.second.size() == nb_runs);
+                    } } } } }
 #endif
+
                 }
                 assert(nb_runs > 0);
                 return v / (nb_runs * (error_max-error_min) * (evals_max-evals_min));
             }
+
+            /** Computes the absolute "hypervolume" of the whole empirical attainement function.
+             *
+             * @warning The nadir reference point is computed automatically as the extremum of all the levels.
+             *          Thus, if you call this function several times on different levels,
+             *          the surfaces will (may) be computed with different nadir points.
+             *          To indicate your own nadir point, use the stat::Volume class directly.
+             * 
+             * @warning This will always compute a metric that should be maximized in order to improve the performance.
+             *          (i.e. *under* the curve for maximization problems, *above* the curve for minimization problems)
+             * 
+             * @ingroup EAF
+             */
+            inline double volume(const EAF& logger)
+            {
+                return volume(logger.optimization_type(), levels(logger));
+            }
             
         } // stat
     } // eaf
