@@ -10,6 +10,8 @@ namespace ioh
     namespace logger
     {
 
+        inline std::string DEFAULT_DOUBLE_FORMAT = "{:.10f}";
+        
         /** @defgroup Properties Properties
          * Accessors to values that are to be logged.
          *
@@ -73,7 +75,7 @@ namespace ioh
 
         public:
             //! Constructor.
-            Property(const std::string &name, const std::string &format = "{:f}") : name_(name), format_(format)
+            Property(const std::string &name, const std::string &format = DEFAULT_DOUBLE_FORMAT) : name_(name), format_(format)
             {
                 assert(!name_.empty());
             }
@@ -134,7 +136,7 @@ namespace ioh
         struct Evaluations : public logger::Property
         {
             //! Constructor.
-            Evaluations(const std::string name = "evaluations", const std::string &format = "{:g}") :
+            Evaluations(const std::string& name = "evaluations", const std::string &format = "{:g}") :
                 logger::Property(name, format)
             {
             }
@@ -149,7 +151,34 @@ namespace ioh
          *
          * @ingroup Properties
          */
-        inline Evaluations evaluations; // Uncomment if one want a library.
+        inline Evaluations evaluations; 
+
+        /** A property that access the cyrrent objective value, without transformation.
+         *
+         * @ingroup Logging
+         */
+        struct RawY : public logger::Property
+        {
+            //! Constructor.
+            RawY(const std::string name = "raw_y", const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
+                logger::Property(name, format)
+            {
+            }
+            //! Main call interface.
+            std::optional<double> operator()(const logger::Info &log_info) const override
+            {
+                if(abs(log_info.raw_y) == std::numeric_limits<double>::infinity()){
+                    return {};
+                }                
+                return std::make_optional(log_info.raw_y);
+            }
+        };
+
+        /** Current function value, without transformation.
+         *
+         * @ingroup Properties
+         */
+        inline RawY raw_y; 
 
         /** A property that access the best value so far, without transformation.
          *
@@ -158,7 +187,7 @@ namespace ioh
         struct RawYBest : public logger::Property
         {
             //! Constructor.
-            RawYBest(const std::string name = "raw_y_best", const std::string &format = "{:f}") :
+            RawYBest(const std::string name = "raw_y_best", const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
                 logger::Property(name, format)
             {
             }
@@ -172,30 +201,7 @@ namespace ioh
          *
          * @ingroup Properties
          */
-        inline RawYBest raw_y_best; // Uncomment if one want a library.
-
-        /** A property that access the current value so far, without transformation.
-         *
-         * @ingroup Logging
-         */
-        struct CurrentY : public logger::Property
-        {
-            //! Constructor.
-            //! Main call interface.
-            CurrentY(const std::string name = "current_y", const std::string &format = "{:f}") :
-                logger::Property(name, format)
-            {
-            }
-            std::optional<double> operator()(const logger::Info &log_info) const override
-            {
-                return std::make_optional(log_info.current.y);
-            }
-        };
-        /** Objective function value for this call, without transformation.
-         *
-         * @ingroup Properties
-         */
-        inline CurrentY current_y; // Uncomment if one want a library.
+        inline RawYBest raw_y_best;         
 
         /** A property that access the current value so far, with transformation.
          *
@@ -205,7 +211,7 @@ namespace ioh
         {
             //! Constructor.
             //! Main call interface.
-            TransformedY(const std::string name = "transformed_y", const std::string &format = "{:f}") :
+            TransformedY(const std::string name = "transformed_y", const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
                 logger::Property(name, format)
             {
             }
@@ -218,7 +224,8 @@ namespace ioh
          *
          * @ingroup Properties
          */
-        inline TransformedY transformed_y; // Uncomment if one want a library.
+        inline TransformedY transformed_y; 
+
         /** A property that access the best value found so far, with transformation.
          *
          * @ingroup Logging
@@ -226,7 +233,7 @@ namespace ioh
         struct TransformedYBest : public logger::Property
         {
             //! Constructor.
-            TransformedYBest(const std::string name = "transformed_y_best", const std::string &format = "{:f}") :
+            TransformedYBest(const std::string name = "transformed_y_best", const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
                 logger::Property(name, format)
             {
             }
@@ -240,7 +247,121 @@ namespace ioh
          *
          * @ingroup Properties
          */
-        inline TransformedYBest transformed_y_best; // Uncomment if one want a library.
+        inline TransformedYBest transformed_y_best; 
+
+        /** A property that access the current value so far, with transformation and constraints applied
+         *
+         * @ingroup Logging
+         */
+        struct CurrentY : public logger::Property
+        {
+            //! Constructor.
+            //! Main call interface.
+            CurrentY(const std::string name = "current_y", const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
+                logger::Property(name, format)
+            {
+            }
+            std::optional<double> operator()(const logger::Info &log_info) const override
+            {
+                return std::make_optional(log_info.y);
+            }
+        };
+        /** Objective function value for this call, without transformation.
+         *
+         * @ingroup Properties
+         */
+        inline CurrentY current_y; 
+
+
+         /** A property that access the current best value so far, with transformation and constraints applied
+         *
+         * @ingroup Logging
+         */
+        struct CurrentBestY : public logger::Property
+        {
+            //! Constructor.
+            //! Main call interface.
+            CurrentBestY(const std::string name = "current_y_best",
+                         const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
+                logger::Property(name, format)
+            {
+            }
+            std::optional<double> operator()(const logger::Info &log_info) const override
+            {
+                return std::make_optional(log_info.y_best);
+            }
+        };
+        /** Objective function value for this call, without transformation.
+         *
+         * @ingroup Properties
+         */
+        inline CurrentBestY current_y_best;
+
+
+        /** A property that accesses the current constraint violation
+         *
+         * @ingroup Logging
+         */
+        struct Violation : public logger::Property
+        {
+            //! Constructor.
+            //! Main call interface.
+            Violation(const std::string name = "violation",
+                      const std::string &format = logger::DEFAULT_DOUBLE_FORMAT,
+                      const size_t ci = 0
+                ) :
+                logger::Property(name, format),
+                ci(ci)
+            {
+            }
+            std::optional<double> operator()(const logger::Info &log_info) const override
+            {
+                return std::make_optional(log_info.violations[ci]);
+            }
+
+            private:
+                //! Index of the constraint to be logged
+                size_t ci;
+        };
+
+        /** Objective function value for this call, without transformation.
+         *
+         * @ingroup Properties
+         */
+        inline Violation violation;
+
+
+        /** A property that accesses the current constraint penalty
+         *
+         * @ingroup Logging
+         */
+        struct Penalty : public logger::Property
+        {
+            //! Constructor.
+            //! Main call interface.
+            Penalty(const std::string name = "penalty", 
+                    const std::string &format = logger::DEFAULT_DOUBLE_FORMAT,
+                    const size_t ci = 0
+                ) :
+                logger::Property(name, format),
+                ci(ci)
+            {
+            }
+            std::optional<double> operator()(const logger::Info &log_info) const override
+            {
+                return std::make_optional(log_info.penalties[ci]);
+            }
+
+        private:
+            //! Index of the constraint to be logged
+            size_t ci;
+        };
+
+        /** Objective function value for this call, without transformation.
+         *
+         * @ingroup Properties
+         */
+        inline Penalty penalty;
 
         /** A property that access a referenced variable.
          *
@@ -262,7 +383,7 @@ namespace ioh
              * @param variable a reference to the logged variable.
              * @param format a fmt::format specification
              */
-            Reference(const std::string name, const T &variable, const std::string &format = "{:f}") :
+            Reference(const std::string name, const T &variable, const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
                 logger::Property(name, format), _variable(variable)
             {
             }
@@ -282,7 +403,7 @@ namespace ioh
          * @ingroup Properties
          */
         template <class T>
-        Reference<T> &reference(const std::string name, const T &variable, const std::string &format = "{:f}")
+        Reference<T> &reference(const std::string name, const T &variable, const std::string &format = logger::DEFAULT_DOUBLE_FORMAT)
         {
             auto p = new Reference<T>(name, variable, format);
             return *p;
@@ -308,7 +429,7 @@ namespace ioh
              * @param variable a pointer to the logged variable.
              * @param format a fmt::format specification
              */
-            Pointer(const std::string name, const T *const variable, const std::string &format = "{:f}") :
+            Pointer(const std::string name, const T *const variable, const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
                 logger::Property(name, format), _variable(variable)
             {
                 assert(variable != nullptr);
@@ -324,11 +445,12 @@ namespace ioh
          *
          * @param name the name of the property.
          * @param variable a pointer to the logged variable.
+         * @param format a string to format the variable when logging. 
          *
          * @ingroup Properties
          */
         template <class T>
-        Pointer<T> &address(const std::string name, const T *const variable, const std::string &format = "{:f}")
+        Pointer<T> &address(const std::string name, const T *const variable, const std::string &format = logger::DEFAULT_DOUBLE_FORMAT)
         {
             auto p = new Pointer<T>(name, variable, format);
             return *p;
@@ -356,7 +478,7 @@ namespace ioh
             //! Typedef for the const ptr ref
             using RefType = ConstPtrType &;
             //! Typedef for the const const ptr ref
-            using ConstRefType = const RefType; // clang++-9 issues a warning for the const having no effect.
+            using ConstRefType = RefType; 
 
             // using Type = const T *const &;
             // We failed to make the above direct declaration work under g++-8 and clang++-9.
@@ -378,14 +500,14 @@ namespace ioh
              * @param ref_ptr_var a reference to a pointer to the logged variable.
              * @param format a fmt::format specification
              */
-            PointerReference(const std::string name, ConstRefType ref_ptr_var, const std::string &format = "{:f}") :
+            PointerReference(const std::string name, ConstRefType ref_ptr_var, const std::string &format = logger::DEFAULT_DOUBLE_FORMAT) :
                 logger::Property(name, format), _ref_ptr_var(ref_ptr_var)
             {
                 if (_ref_ptr_var != nullptr)
                 {
-                    IOH_DBG(debug, "PointerReference " << name << " @ " << ref_ptr_var << " == " << static_cast<double>(*ref_ptr_var));
+                    // IOH_DBG(debug, "PointerReference "  name << " @ " << ref_ptr_var << " == " << static_cast<double>(*ref_ptr_var));
                 }
-                IOH_DBG(debug, "PointerReference " << name << " @ " << ref_ptr_var << " == nullptr");
+                // IOH_DBG(debug, "PointerReference " << name << " @ " << ref_ptr_var << " == nullptr");
             }
 
             //! Main call interface.
@@ -393,10 +515,10 @@ namespace ioh
             {
                 if (_ref_ptr_var != nullptr)
                 {
-                    IOH_DBG(debug, "PointerReference " << name() << " @ " << _ref_ptr_var << " == " << static_cast<double>(*_ref_ptr_var));
+                    // IOH_DBG(debug, "PointerReference " << name() << " @ " << _ref_ptr_var << " == " << static_cast<double>(*_ref_ptr_var));
                     return std::make_optional(static_cast<double>(*_ref_ptr_var));
                 }
-                IOH_DBG(debug, "PointerReference " << name() << " @ " << _ref_ptr_var << " == nullopt");
+                // IOH_DBG(debug, "PointerReference " << name() << " @ " << _ref_ptr_var << " == nullopt");
                 return std::nullopt;
             }
 
@@ -422,7 +544,7 @@ namespace ioh
          */
         template <class T>
         PointerReference<T> &pointer(const std::string name, const T *const &variable,
-                                     const std::string &format = "{:f}")
+                                     const std::string &format = logger::DEFAULT_DOUBLE_FORMAT)
         {
             auto p = new PointerReference<T>(name, variable, format);
             return *p;
