@@ -127,7 +127,7 @@ namespace ioh
             /** Propagate the reset event to all managed triggers. */
             virtual void reset() override
             {
-                IOH_DBG(debug, "reset triggers")
+                IOH_DBG(debug, "reset triggers in Set");
                 for (auto &trigger : triggers_)
                 {
                     trigger.get().reset();
@@ -171,6 +171,7 @@ namespace ioh
             /** Triggered if ANY the managed triggers are triggered. */
             virtual bool operator()(const logger::Info &log_info, const problem::MetaData &pb_info) override
             {
+                IOH_DBG(debug, "trigger Any called");
                 assert(!triggers_.empty());
                 for (auto &trigger : triggers_)
                 {
@@ -216,6 +217,7 @@ namespace ioh
             /** Triggered if ALL the managed triggers are triggered. */
             virtual bool operator()(const logger::Info &log_info, const problem::MetaData &pb_info) override
             {
+                IOH_DBG(debug, "trigger All called");
                 assert(triggers_.size() > 0);
                 for (auto &trigger : triggers_)
                 {
@@ -261,73 +263,13 @@ namespace ioh
          *
          * @ingroup Triggering
          */
-        class OnImprovement : public logger::Trigger
+        struct OnImprovement : logger::Trigger
         {
-        protected:
-            //! Local memory of the best value.
-            double _best;
-            //! Current problem type.
-            common::OptimizationType _type;
-
-            //! State management flag.
-            // Avoids being dependent on a problem at construction,
-            // to the expense of a test at each call.
-            bool _has_type;
-
-        public:
-            //! Constructor.
-            OnImprovement() : _has_type(false) { reset(); }
-
-            /**
-             * @brief Construct a new On Improvement object
-             * 
-             * @param best Local memory of the best value.
-             * @param type Current problem type.
-             */
-            OnImprovement(double best, common::OptimizationType type) : _best(best), _type(type), _has_type(true) {}
-
-            //! Accessor for _best
-            double best() const { return _best; }
-
-            //! Accessor for _type
-            common::OptimizationType type() const { return _type; }
-
-
-            //! Main call interface.
-            bool operator()(const logger::Info &log_info, const problem::MetaData &pb_info) override
+            bool operator()(const logger::Info &log_info, const problem::MetaData&) override
             {
-                if (not _has_type)
-                {
-                    _type = pb_info.optimization_type;
-                    if (_type == common::OptimizationType::Minimization)
-                    {
-                        IOH_DBG(debug, "reconfigure problem type as minimization")
-                        _best = std::numeric_limits<double>::infinity();
-                    }
-                    else
-                    {
-                        IOH_DBG(debug, "reconfigure problem type as maximization")
-                        _best = -std::numeric_limits<double>::infinity();
-                    }
-                    _has_type = true;
-                }
-                // We do not use logger::Info::transformed_y_best below,
-                // because all fields of logger::Info are updated before the trigger see them.
-                // That would force to test for equality to trigger on improvement,
-                // and we only want to trigger on strict inequality.
-                assert(_has_type);
-                if (common::compare_objectives(log_info.transformed_y, _best, _type))
-                {
-                    _best = log_info.transformed_y;
-                    IOH_DBG(debug, "triggered on improvement by " << log_info.transformed_y << " / " << _best)
-                    return true;
-                }
-                IOH_DBG(xdebug, "not triggered on improvement by " << log_info.transformed_y << " / " << _best)
-                return false;
+                IOH_DBG(debug, "trigger OnImprovement called: " << log_info.has_improved);
+                return log_info.has_improved;
             }
-
-            //! Forget previous state.
-            void reset() override { _has_type = false; }
         };
         /** Do log only if the transformed best objective function value found so far has strictly improved.
          *
@@ -358,13 +300,14 @@ namespace ioh
             }
             //! Accessor for _interval
             size_t interval() const { return _interval; }
-            
+
             //! Accessor for _starting_at
             size_t starting_at() const { return _starting_at; }
 
             //! Main call interface.
             bool operator()(const logger::Info &log_info, const problem::MetaData &) override
             {
+                IOH_DBG(debug, "trigger Each called");
                 if ((log_info.evaluations - _starting_at) % _interval == 0)
                 {
                     IOH_DBG(debug, "each triggered " << log_info.evaluations)
@@ -409,11 +352,13 @@ namespace ioh
              */
             At(const std::set<size_t> time_points) : _time_points(time_points) {}
 
-            std::set<size_t> time_points() const {return _time_points;}
+            //! Return the time points when to log
+            std::set<size_t> time_points() const { return _time_points; }
 
             //! Main call interface.
             bool operator()(const logger::Info &log_info, const problem::MetaData &) override
             {
+                IOH_DBG(debug, "trigger At called");
                 if (matches(log_info.evaluations))
                 {
                     IOH_DBG(debug, "triggered at " << log_info.evaluations)
@@ -425,8 +370,6 @@ namespace ioh
                     return false;
                 }
             }
-
-
         };
         /** Do log at the given number of evaluations.
          *
@@ -482,11 +425,12 @@ namespace ioh
 #endif
             }
             //! Accessor for _time_ranges
-            std::set<std::pair<size_t, size_t>> time_ranges() const {return _time_ranges;}
+            std::set<std::pair<size_t, size_t>> time_ranges() const { return _time_ranges; }
 
             //! Main call interface.
             bool operator()(const logger::Info &log_info, const problem::MetaData &) override
             {
+                IOH_DBG(debug, "trigger During called");
                 if (matches(log_info.evaluations))
                 {
                     IOH_DBG(debug, "triggered during " << log_info.evaluations)
