@@ -5,5 +5,10 @@
 #include "problem/bbob.hpp"
 #include "problem/pbo.hpp"
 
-#include "problem/python.hpp"
 #include "problem/wmodel.hpp"
+
+#ifdef USING_MKLANDSCAPE
+#include "problem/mklandscape/cliqueTreeC.hpp"
+#endif
+
+#include "problem/submodular.hpp"
