@@ -12,28 +12,22 @@ namespace ioh::problem::bbob
         //! Evaluation method
         double evaluate(const std::vector<double> &x) override
         {
+            using namespace transformation::objective;
             auto result =  0.0 ;
             for (auto i = 0; i < meta_data_.n_variables; ++i)
-                result += x.at(i) * x.at(i) * (1. + 9999.0 * (objective_.x.at(i) * x.at(i) > 0.0));
-            return result;
+                result += x.at(i) * x.at(i) * (1. + 9999.0 * (optimum_.x.at(i) * x.at(i) > 0.0));
+            return pow(oscillate(result), .9);
         }
         
         //! Variables transformation method
         std::vector<double> transform_variables(std::vector<double> x) override
         {
             using namespace transformation::variables;
-            subtract(x, objective_.x);
+            subtract(x, optimum_.x);
             affine(x, transformation_state_.second_transformation_matrix, transformation_state_.transformation_base);
             return x;
         }
 
-        //! Objectives transformation method
-        double transform_objectives(const double y) override
-        {
-            using namespace transformation::objective;
-            return shift(pow(oscillate(y), .9), objective_.y);
-        }
-
     public:
         /**
          * @brief Construct a new Attractive Sector object
