@@ -1,9 +1,7 @@
-"""Python specific functions for IOH package
-
-TODO: best far precision
-TODO: Rename Integer -> Discrete
+"""Python interface of IOH package. Includes several ease-of-use routines not available in C++. 
 """
 
+from heapq import merge
 import os
 import math
 import warnings
@@ -12,31 +10,37 @@ import multiprocessing
 import typing
 import shutil
 import copy
-
-try:
-    from .iohcpp import (
-        problem,
-        suite,
-        logger,
-        OptimizationType,
-        RealSolution,
-        IntegerSolution,
-        IntegerConstraint,
-        RealConstraint,
-        RealState,
-        IntegerState,
-        MetaData,
-        LogInfo,
-    )
-except ModuleNotFoundError:
-    raise ModuleNotFoundError("No module named ioh")
+import json
+ 
+from .iohcpp import (
+    problem,
+    suite, 
+    logger,
+    OptimizationType,
+    RealSolution,
+    IntegerSolution,
+    IntegerBounds,
+    RealBounds,
+    IntegerConstraint,
+    RealConstraint,
+    ConstraintEnforcement,
+    RealState, 
+    IntegerState,
+    MetaData,
+    LogInfo,
+)
 
 
 ProblemType = typing.Union[problem.Real, problem.Integer]
+VariableType = typing.Union[int, float]
+ObjectiveType = typing.List[VariableType]
 
 
 def get_problem(
-    fid: typing.Union[int, str], iid: int, dim: int, problem_type: str = "Real"
+    fid: typing.Union[int, str],
+    instance: int = 1,
+    dimension: int = 5,
+    problem_type: str = "Real",
 ) -> ProblemType:
     """Instantiate a problem based on its function ID, dimension, instance and suite
 
@@ -44,10 +48,10 @@ def get_problem(
     ----------
     fid: int or str
         The function ID of the problem in the suite, or the name of the function as string
-    dim: int
-        The dimension (number of variables) of the problem
-    iid: int
+    instance: int
         The instance ID of the problem
+    dimension: int
+        The dimension (number of variables) of the problem
     problem_type: str
         Which suite the problem is from. Either 'BBOB' or 'PBO' or 'Real' or 'Integer'
         Only used if fid is an int.
@@ -61,24 +65,19 @@ def get_problem(
         )
         and fid in [21, 23, "IsingTriangular", "NQueens"]
     ):
-        if not math.sqrt(dim).is_integer():
+        if not math.sqrt(dimension).is_integer():
             raise ValueError(
                 "For this function, the dimension needs to be a perfect square!"
             )
-
-    bbob_fns = (
-        getattr(problem, "BBOB").problems.values()
-        | getattr(problem, "BBOB").problems.keys()
-    )
     if (
         problem_type
         in (
             "BBOB",
             "Real",
         )
-        and fid in bbob_fns
+        and fid in range(1, 25)
     ):
-        if not dim >= 2:
+        if not dimension >= 2:
             raise ValueError("For BBOB functions the minimal dimension is 2")
 
     base_problem = getattr(problem, problem_type)
@@ -87,12 +86,110 @@ def get_problem(
             raise ValueError(
                 f"{fid} is not registered for problem type: {problem_type}"
             )
-        return base_problem.create(fid, iid, dim)
+        return base_problem.create(fid, instance, dimension)
 
     raise ValueError(f"Problem type {problem_type} is not supported")
 
 
+def wrap_problem(
+    function: typing.Callable[[ObjectiveType], float],
+    name: str,
+    problem_type: str,
+    dimension: int = 5,
+    instance: int = 1,
+    optimization_type: OptimizationType = OptimizationType.MIN,
+    lb: VariableType = None,
+    ub: VariableType = None,
+    transform_variables: typing.Callable[[ObjectiveType, int], ObjectiveType] = None,
+    transform_objectives: typing.Callable[[float, int], float] = None,
+    calculate_objective: typing.Callable[
+        [int, int], typing.Union[IntegerSolution, RealSolution]
+    ] = None,
+    constraints: typing.List[typing.Union[IntegerConstraint, RealConstraint]] = None
+
+) -> ProblemType:
+    """Function to wrap a callable as an ioh function
+
+    Parameters
+    ----------
+    function: fn(x: list) -> float
+        The callable to wrap
+    name: str
+        The name of the function. This can be used to create new instances of this function.
+        Note, when not using unique names, this will override the previously wrapped functions.
+    problem_type: str
+        The type of the problem, accepted types are: 'Real' and 'Integer'
+    dimension: int
+        The dimension (number of variables) of the problem
+    instance: int
+        The instance ID of the problem
+    optimization_type: OptimizationType
+        The type of optimization to do, maximization or minimization
+    lb: [int, float]
+        The lower bound of the constraint, should be the same type as problem_type. When left to None, this will set
+        to the lowest possible value for that type.
+    ub: [int, float]
+        The upper bound of the constraint, should be the same type as problem_type. When left to None, this will set
+        to the highest possible value for that type.
+    transform_variables: fn(x: list) -> list
+        A function to transform the elements of x, prior to calling the function.
+    transform_objectives: fn(y: float) -> float
+        A function to tranform the float value of y, after callling the function.
+    calculate_objective: fn(instance, dimension) -> Solution | (x, y)
+        A function to calculate the global optimum of the function. This function gets a dimension and instance id,
+        and should return either a Solution objective(IntegerSolution or RealSolution) or a tuple giving the
+        x and y values for the global optimum. Where x is the search space representation and y the target value.
+    constraints: list[IntegerConstraint | RealConstraint]
+        The constraints applied to the problem
+    """
+
+    if problem_type == "Integer":
+        wrapper = problem.wrap_integer_problem
+    elif problem_type == "Real":
+        wrapper = problem.wrap_real_problem
+    else:
+        raise ValueError(
+            f"Problem type {problem_type} is not supported. Please select one of ('Integer', 'Real')"
+        )
+
+    wrapper(
+        function,
+        name,
+        optimization_type,
+        lb,
+        ub,
+        transform_variables,
+        transform_objectives,
+        calculate_objective,
+        [] if constraints is None else constraints
+    )
+    return get_problem(name, instance, dimension, problem_type)
+
+
+def get_problem_id(problem_name: str, problem_type: str) -> int: 
+    '''Get the problem id corresponding to a problem name of a given type
+    
+    Parameters
+    ----------
+    problem_name:str
+        The name of the problem
+    problem_type:str
+        The type of the problem
+    
+    Returns
+    -------
+    int: the id of the problem
+    '''
+    if problem_type in ("BBOB", "PBO", "Integer", "Real"):
+        return {v: k for k, v in getattr(problem, problem_type).problems.items()}.get(
+            problem_name
+        )
+    raise ValueError(f"Problem type {problem_type} is not supported")
+
+
 class Experiment:
+    '''Class to help easily setup benchmarking experiments. '''
+
     def __init__(
         self,
         algorithm: typing.Any,
@@ -100,7 +197,7 @@ class Experiment:
         iids: typing.List[int],
         dims: typing.List[int],
         reps: int = 1,
-        problem_type: str = "BBOB",
+        problem_type: str = "Real",
         njobs: int = 1,
         logged: bool = True,
         logger_triggers: typing.List[logger.trigger.Trigger] = [
@@ -120,6 +217,8 @@ class Experiment:
         merge_output: bool = True,
         zip_output: bool = True,
         remove_data: bool = False,
+        enforce_bounds: bool = False,
+        old_logger: bool = True
     ):
         """
         Parameters
@@ -158,7 +257,7 @@ class Experiment:
                 A name for the algorithm. This is used in the log files.
             algorithm_info: str = ""
                 Optional information, additional information used in log files
-            optimization_type: OptimizationType = OptimizationType.Minimization
+            optimization_type: OptimizationType = OptimizationType.MIN
                 The type of optimization
             store_positions: bool = False
                 Whether to store the x-positions in the data-files
@@ -214,6 +313,8 @@ class Experiment:
         self.merge_output = merge_output
         self.zip_output = zip_output
         self.remove_data = remove_data
+        self.enforce_bounds = enforce_bounds
+        self.old_logger = old_logger
 
         if os.path.isdir(self.logger_root) and self.merge_output:
             warnings.warn(
@@ -248,13 +349,18 @@ class Experiment:
         if self.logged:
             logger_params = copy.deepcopy(self.logger_params)
             logger_params["folder_name"] += f"-tmp-{ii}"
-            l = logger.Analyzer(**logger_params)
+            
+            logger_cls = logger.old.Analyzer if self.old_logger else logger.Analyzer
+            l = logger_cls(**logger_params) 
             l.set_experiment_attributes(self.experiment_attributes)
             l.add_run_attributes(algorithm, self.run_attributes)
             l.watch(algorithm, self.logged_attributes)
             l.reset()
             p.attach_logger(l)
-            
+
+        if self.enforce_bounds:
+            p.enforce_bounds()
+
         self.apply(algorithm, p)
 
     def apply(self, algorithm: any, problem: ProblemType):
@@ -277,14 +383,20 @@ class Experiment:
             An optional name of the the newly added function.
         """
 
-        if self.problem_type in ("PBO", "Integer"):
-            p = problem.wrap_integer_problem(p, name or "CustomProblem", **kwargs)
-            self.problem_type = "Integer"
+        name = name or "CustomProblem"
+
+        if self.problem_type == "Integer":
+            problem.wrap_integer_problem(p, name, **kwargs)
+        elif self.problem_type == "Real":
+            problem.wrap_real_problem(p, name, **kwargs)
         else:
-            p = problem.wrap_real_problem(p, name or "CustomProblem", **kwargs)
-            self.problem_type = "Real"
-        
-        self.fids.append(p.meta_data.problem_id)
+            warnings.warn(
+                "Adding custom problems is only allowed for problem_type in ('Integer', 'Real'). "
+                "The added problem is ignored."
+            )
+            return
+
+        self.fids.append(get_problem_id(name, self.problem_type))
 
     def merge_output_to_single_folder(self, prefix: str, target_folder: str):
         """Merges all ioh data folders into a single folder, having the same folder name prefix
@@ -297,6 +409,42 @@ class Experiment:
             The target folder, i.e. the folder with the final output
         """
 
+
+        def merge_info_file(target, source):                        
+            ext = ".info" if self.old_logger else ".json"
+
+            if not (target.endswith(ext) and source.endswith(ext)):
+                raise RuntimeError("Merging output with incompatible folders")
+
+            has_file = os.path.isfile(target)
+            if not has_file:
+                return os.rename(source, target)
+            with open(source) as info_in:
+                if self.old_logger:
+                    with open(target, "a+") as info_out:        
+                        info_out.write("\n")
+                        for line in info_in:
+                            info_out.write(line)
+                else:
+                    with open(target, "r+") as info_out:
+                        data_out = json.loads(info_out.read())
+                        data_in = json.loads(info_in.read())
+                        for scen in data_in['scenarios']:
+                            try:
+                                scen_out, *_ = [s for s in data_out['scenarios'] if s['dimension'] == scen['dimension']]
+                                scen_out['runs'].extend(scen['runs'])
+                            except:
+                                data_out['scenarios'].append(scen)
+                        info_out.seek(0)
+                        info_out.write(json.dumps(data_out, indent=4))
+                        info_out.truncate()
+            os.remove(source)
+                    
+
+        def file_to_dir(path):
+            root, dirname = os.path.split(os.path.splitext(path)[0])
+            return os.path.join(root, dirname.replace("IOHprofiler", "data"))
+
         root = os.path.dirname(self.logger_root)
 
         for folder_name in os.listdir(root):
@@ -313,25 +461,12 @@ class Experiment:
                     source = os.path.join(folder, info_file)
                     if not os.path.isfile(source):
                         continue
-
-                    if not info_file.endswith(".info"):
-                        raise RuntimeError("Merging output with incompatible folders")
-
+                    
                     target = os.path.join(target_folder, info_file)
-                    target_exists = os.path.isfile(target)
-                    source_dat_folder = os.path.splitext(source)[0].replace(
-                        "IOHprofiler", "data"
-                    )
-                    target_dat_folder = os.path.splitext(target)[0].replace(
-                        "IOHprofiler", "data"
-                    )
+                    merge_info_file(target, source)
 
-                    with open(source) as info_in, open(target, "a+") as info_out:
-                        if target_exists:
-                            info_out.write("\n")
-                        for line in info_in:
-                            info_out.write(line)
-                    os.remove(source)
+                    source_dat_folder = file_to_dir(source)
+                    target_dat_folder = file_to_dir(target)
 
                     os.makedirs(target_dat_folder, exist_ok=True)
                     for dat_file in os.listdir(source_dat_folder):
@@ -356,7 +491,7 @@ class Experiment:
             target = root + f"-{idx}"
             idx += 1
 
-        os.rename(root + "-tmp-1", target)
+        os.rename(self.logger_root + "-tmp-1", target)
         self.merge_output_to_single_folder(root + "-tmp", target)
 
     def run(self):
@@ -375,7 +510,6 @@ class Experiment:
             for job in iterator:
                 self.evaluate(*job)
 
-
         self.merge_tmp_folders()
 
         if self.merge_output:
@@ -390,3 +524,12 @@ class Experiment:
             shutil.rmtree(self.logger_root)
 
         return self
+
+
+__all__ = (
+    "get_problem",
+    "wrap_problem",
+    "get_problem_id",
+    "Experiment"
+)
+
