@@ -9,6 +9,23 @@
 namespace py = pybind11;
 using namespace ioh;
 
+// Trampoline 
+struct AbstractProperty : logger::Property
+{
+    AbstractProperty(const std::string& name): logger::Property(name) {}
+
+    std::string call_to_string(const logger::Info &log_info, const std::string &nan = "") const override
+    {
+        PYBIND11_OVERRIDE(std::string, logger::Property, call_to_string, log_info, nan);
+    }
+
+    std::optional<double> operator()(const logger::Info & info) const override
+    {
+        PYBIND11_OVERRIDE_PURE_NAME(std::optional<double>, logger::Property, "__call__", operator(), info);
+    }
+};
+
+// Python spec. implementation
 class PyProperty : public logger::Property
 {
     const py::object container_;
@@ -25,55 +42,48 @@ public:
     std::optional<double> operator()(const logger::Info &) const override
     {
         if (py::hasattr(container_, attribute_.c_str()))
-            return std::make_optional<double>(PyFloat_AsDouble(container_.attr(attribute_.c_str()).ptr()));
+        {
+            auto pyobj = container_.attr(attribute_.c_str()).ptr();
+            if (pyobj != Py_None)
+                return std::make_optional<double>(PyFloat_AsDouble(pyobj));
+        }
         return {};
     }
 };
 
+// Trampoline 
+struct AbstractWatcher: logger::Watcher {
 
+    using logger::Watcher::Watcher;
+
+    void attach_problem(const problem::MetaData& problem) override {
+        PYBIND11_OVERRIDE(void, logger::Watcher, attach_problem, problem);
+    }
+    void attach_suite(const std::string& suite_name) override {
+        PYBIND11_OVERRIDE_PURE(void, logger::Watcher, attach_suite, suite_name);
+    }
+    void call(const logger::Info& log_info) override {
+        PYBIND11_OVERRIDE_PURE_NAME(void, logger::Watcher, "__call__", call, log_info);
+    }
+};
+
+// Python spec. implementation
 template <typename WatcherType>
 class PyWatcher : public WatcherType
 {
 protected:
-    bool alive = true;
-    std::vector<PyProperty *> property_ptrs_;
+    std::vector<std::unique_ptr<PyProperty>> property_ptrs_;
 
 public:
-    template <typename... Args>
-    PyWatcher(Args &&...args) : WatcherType(std::forward<Args>(args)...)
-    {
-        py::module::import("atexit").attr("register")(py::cpp_function{[self = this]() -> void {
-            // type-pun alive bool in order to check if is still a boolean 1, if so, delete.
-            // in some cases this might cause a segfault, only happens in a very small prob. (1/MAX_INT)
-            int alive_int = (int)(*(char *)(&self->alive));
-            if (alive_int == 1)
-            {
-                self->close();
-            }
-        }});
-    }
-
-    void close() override
-    {
-        if (alive)
-        {
-            WatcherType::close();
-            alive = false;
-            for (auto ptr : property_ptrs_)
-                delete ptr;
-            property_ptrs_.clear();
-        }
-    }
-
-    virtual ~PyWatcher() { close(); }
+    using WatcherType::WatcherType;
 
     void watch(logger::Property &property) override { PYBIND11_OVERRIDE(void, WatcherType, watch, property); }
 
     void watch(const py::object &container, const std::string &attribute)
     {
-        auto *p = new PyProperty(container, attribute);
+        auto p = std::make_unique<PyProperty>(container, attribute); 
         watch(*p);
-        property_ptrs_.push_back(p);
+        property_ptrs_.push_back(std::move(p));
     }
 
     void watch(const py::object &container, const std::vector<std::string> &attributes)
@@ -82,8 +92,6 @@ public:
             watch(container, attr);
     }
 
-    void log(const logger::Info &log_info) override { PYBIND11_OVERRIDE(void, WatcherType, log, log_info); }
-
     void attach_problem(const problem::MetaData &problem) override
     {
         PYBIND11_OVERRIDE(void, WatcherType, attach_problem, problem);
@@ -97,31 +105,27 @@ public:
     void call(const logger::Info &log_info) override { PYBIND11_OVERRIDE(void, WatcherType, call, log_info); }
 };
 
-class PyAnalyzer : public PyWatcher<logger::Analyzer>
+// Python spec. implementation
+template<typename A>
+class PyAnalyzer : public PyWatcher<A>
 {
-    std::vector<double *> double_ptrs_;
-    std::vector<PyProperty *> prop_ptrs_;
+    std::vector<std::unique_ptr<double>> double_ptrs_;
+    std::vector<std::unique_ptr<PyProperty>> prop_ptrs_;
 
 public:
-    using Analyzer = PyWatcher<logger::Analyzer>;
-    using Analyzer::Analyzer;
+    using AnalyzerType = PyWatcher<A>;
+    using AnalyzerType::AnalyzerType;
 
     virtual void close() override
     {
-        if (alive)
-        {
-            clear_ptrs();
-            Analyzer::close();
-        }
+        AnalyzerType::close();
     }
 
-    virtual ~PyAnalyzer() { close(); }
-
     void add_run_attribute_python(const py::object &container, const std::string &name)
     {
-        auto *p = new PyProperty(container, name);
+        auto p = std::make_unique<PyProperty>(container, name);
         add_run_attribute_python(name, (*p)(logger::Info{}).value());
-        prop_ptrs_.push_back(p);
+        prop_ptrs_.push_back(std::move(p));
     }
 
     void add_run_attributes_python(const py::object &container, const std::vector<std::string> &attributes)
@@ -132,36 +136,25 @@ public:
 
     void add_run_attribute_python(const std::string &name, double value)
     {
-        double *ptr = new double(value);
-        Analyzer::add_run_attribute(name, ptr);
-        double_ptrs_.push_back(ptr);
+        auto ptr = std::make_unique<double>(value);
+        AnalyzerType::add_run_attribute(name, ptr.get());
+        double_ptrs_.push_back(std::move(ptr));
     }
 
-    void set_run_attribute_python(const std::string &name, double value) { *(attributes_.run.at(name)) = value; }
+    void set_run_attribute_python(const std::string &name, double value) { *(this->attributes_.run.at(name)) = value; }
 
     void set_run_attributes_python(const std::map<std::string, double> &attributes)
     {
-        clear_ptrs();
+        double_ptrs_.clear();
         for (auto &[key, value] : attributes)
             add_run_attribute_python(key, value);
     }
 
-    void clear_ptrs()
-    {
-        for (auto ptr : double_ptrs_)
-            delete ptr;
-
-        for (auto ptr : prop_ptrs_)
-            delete ptr;
-        double_ptrs_.clear();
-        prop_ptrs_.clear();
-    }
-
     void attach_problem(const problem::MetaData &problem) override
     {
-        for (auto ptr : prop_ptrs_)
+        for (auto& ptr : prop_ptrs_)
             set_run_attribute_python(ptr->name(), (*ptr)(logger::Info{}).value());
-        Analyzer::attach_problem(problem);
+        AnalyzerType::attach_problem(problem);
     }
 };
 
@@ -169,35 +162,53 @@ void define_triggers(py::module &m)
 {
     py::module t = m.def_submodule("trigger");
 
-    py::class_<logger::Trigger, std::shared_ptr<logger::Trigger>>(t, "Trigger")
+    py::class_<logger::Trigger, std::shared_ptr<logger::Trigger>>(t, "Trigger", "Base class for all Triggers")
         .def("__call__", &logger::Trigger::operator())
         .def("reset", &logger::Trigger::reset);
 
-    py::class_<trigger::Always, logger::Trigger, std::shared_ptr<trigger::Always>>(t, "Always")
+    py::class_<trigger::Always, logger::Trigger, std::shared_ptr<trigger::Always>>(
+        t, "Always", "Trigger that always evaluates to true")
         .def(py::init<>())
         .def(py::pickle([](const trigger::Always &) { return py::make_tuple(); },
                         [](py::tuple) { return trigger::Always{}; }));
 
     t.attr("ALWAYS") = py::cast(trigger::always);
 
-    py::class_<trigger::OnImprovement, logger::Trigger, std::shared_ptr<trigger::OnImprovement>>(t, "OnImprovement")
+    py::class_<trigger::OnImprovement, logger::Trigger, std::shared_ptr<trigger::OnImprovement>>(
+        t, "OnImprovement", "Trigger that evaluates to true when improvement of the objective function is observed")
         .def(py::init<>())
-        .def(py::pickle([](const trigger::OnImprovement &t) { return py::make_tuple(t.best(), t.type()); },
-                        [](py::tuple t) {
-                            return trigger::OnImprovement{t[0].cast<double>(), t[1].cast<common::OptimizationType>()};
-                        }));
+        .def(py::pickle([](const trigger::OnImprovement &) { return py::make_tuple(); },
+                        [](py::tuple) { return trigger::OnImprovement{}; }));
 
     ;
     t.attr("ON_IMPROVEMENT") = py::cast(trigger::on_improvement);
 
     py::class_<trigger::At, logger::Trigger, std::shared_ptr<trigger::At>>(t, "At")
-        .def(py::init<std::set<size_t>>(), py::arg("time_points"))
+        .def(py::init<std::set<size_t>>(), py::arg("time_points"),
+             R"pbdoc(
+                Trigger that evaluates to true at specific time points
+
+                Parameters
+                ----------
+                time_points: list[int]
+                    The time points at which to trigger
+            )pbdoc")
         .def_property_readonly("time_points", &trigger::At::time_points)
         .def(py::pickle([](const trigger::At &t) { return py::make_tuple(t.time_points()); },
                         [](py::tuple t) { return trigger::At{t[0].cast<std::set<size_t>>()}; }));
 
     py::class_<trigger::Each, logger::Trigger, std::shared_ptr<trigger::Each>>(t, "Each")
-        .def(py::init<size_t, size_t>(), py::arg("interval"), py::arg("starting_at") = 0)
+        .def(py::init<size_t, size_t>(), py::arg("interval"), py::arg("starting_at") = 0,
+             R"pbdoc(
+                Trigger that evaluates to true at a given interval
+
+                Parameters
+                ----------
+                interval: int
+                    The interval at which to trigger
+                starting_at: int
+                    The starting time of the interval, defaults to 0.
+            )pbdoc")
         .def_property_readonly("interval", &trigger::Each::interval)
         .def_property_readonly("starting_at", &trigger::Each::starting_at)
         .def(py::pickle([](const trigger::Each &t) { return py::make_tuple(t.interval(), t.starting_at()); },
@@ -206,87 +217,99 @@ void define_triggers(py::module &m)
                         }));
 
     py::class_<trigger::During, logger::Trigger, std::shared_ptr<trigger::During>>(t, "During")
-        .def(py::init<std::set<std::pair<size_t, size_t>>>(), py::arg("time_ranges"))
+        .def(py::init<std::set<std::pair<size_t, size_t>>>(), py::arg("time_ranges"),
+             R"pbdoc(
+                Trigger that evaluates to true during a given interval.
+
+                Parameters
+                ----------
+                time_ranges: set[tuple[int, int]]
+                    The ranges/intervals at which the log the data.                    
+            )pbdoc"
+
+             )
         .def_property_readonly("time_ranges", &trigger::During::time_ranges)
         .def(py::pickle([](const trigger::During &t) { return py::make_tuple(t.time_ranges()); },
                         [](py::tuple t) { return trigger::During{t[0].cast<std::set<std::pair<size_t, size_t>>>()}; }));
 }
 
+template<typename P>
+void define_property(py::module &m, std::string name, P predef){
+
+    py::class_<P, logger::Property, std::shared_ptr<P>>(m, name.c_str(), py::buffer_protocol())
+        .def(py::init<std::string, std::string>(), py::arg("name"), py::arg("format"),
+             ("Property which tracks the " + name).c_str())
+        .def(py::pickle([](const P &t) { return py::make_tuple(t.name(), t.format()); },
+                        [](py::tuple t) {
+                            return P{t[0].cast<std::string>(), t[1].cast<std::string>()};
+                        }));
+    
+    std::transform(name.begin(), name.end(), name.begin(), ::toupper);
+    m.attr(name.c_str()) = py::cast(predef);
+}
+
 void define_properties(py::module &m)
 {
     py::module t = m.def_submodule("property");
 
-    py::class_<logger::Property, std::shared_ptr<logger::Property>>(t, "AbstractProperty")
+    py::class_<logger::Property, AbstractProperty, std::shared_ptr<logger::Property>>(t, "AbstractProperty",
+                                                                                      "Base class for all Properties")
+        .def(py::init<std::string>())
         .def("__call__", &logger::Property::operator())
         .def("name", &logger::Property::name)
         .def("call_to_string", &logger::Property::call_to_string);
 
 
     py::class_<PyProperty, logger::Property, std::shared_ptr<PyProperty>>(t, "Property")
-        .def(py::init<py::object, std::string>(), py::arg("container"), py::arg("attribute"))
+        .def(py::init<py::object, std::string>(), py::arg("container"), py::arg("attribute"),
+             R"pbdoc(
+                Wrapper for properties with a Python container
+
+                Parameters
+                ----------
+                container: object
+                    The object which contains the property to be logged
+                attribute: str
+                    The name of the property to be logger, must be an attribute on container          
+            )pbdoc"
+
+             )
+        .def("call_to_string", &PyProperty::call_to_string)
         .def(py::pickle([](const PyProperty &t) { return py::make_tuple(t.container(), t.name()); },
                         [](py::tuple t) {
                             return PyProperty{t[0].cast<py::object>(), t[1].cast<std::string>()};
                         }));
 
-    py::class_<watch::Evaluations, logger::Property, std::shared_ptr<watch::Evaluations>>(t, "Evaluations")
-        .def(py::init<std::string, std::string>())
-        .def(py::pickle([](const watch::Evaluations &t) { return py::make_tuple(t.name(), t.format()); },
-                        [](py::tuple t) {
-                            return watch::Evaluations{t[0].cast<std::string>(), t[1].cast<std::string>()};
-                        }));
-    t.attr("EVALUATIONS") = py::cast(watch::evaluations);
+    define_property<watch::Evaluations>(t, "Evaluations", watch::evaluations);
 
-    py::class_<watch::RawYBest, logger::Property, std::shared_ptr<watch::RawYBest>>(t, "RawYBest")
-        .def(py::init<std::string, std::string>())
-        .def(py::pickle([](const watch::RawYBest &t) { return py::make_tuple(t.name(), t.format()); },
-                        [](py::tuple t) {
-                            return watch::RawYBest{t[0].cast<std::string>(), t[1].cast<std::string>()};
-                        }));
-    t.attr("RAW_Y_BEST") = py::cast(watch::raw_y_best);
+    define_property<watch::RawY>(t, "RawY", watch::raw_y);
+    define_property<watch::RawYBest>(t, "RawYBest", watch::raw_y_best);
 
-    py::class_<watch::CurrentY, logger::Property, std::shared_ptr<watch::CurrentY>>(t, "CurrentY")
-        .def(py::init<std::string, std::string>())
-        .def(py::pickle([](const watch::CurrentY &t) { return py::make_tuple(t.name(), t.format()); },
-                        [](py::tuple t) {
-                            return watch::CurrentY{t[0].cast<std::string>(), t[1].cast<std::string>()};
-                        }));
-    t.attr("CURRENT_Y_BEST") = py::cast(watch::current_y);
+    define_property<watch::TransformedY>(t, "TransformedY", watch::transformed_y);
+    define_property<watch::TransformedYBest>(t, "TransformedYBest", watch::transformed_y_best);
 
-    py::class_<watch::TransformedY, logger::Property, std::shared_ptr<watch::TransformedY>>(t, "TransformedY")
-        .def(py::init<std::string, std::string>())
-        .def(py::pickle([](const watch::TransformedY &t) { return py::make_tuple(t.name(), t.format()); },
-                        [](py::tuple t) {
-                            return watch::TransformedY{t[0].cast<std::string>(), t[1].cast<std::string>()};
-                        }));
-    t.attr("TRANSFORMED_Y") = py::cast(watch::transformed_y);
+    define_property<watch::CurrentY>(t, "CurrentY", watch::current_y);
+    define_property<watch::CurrentBestY>(t, "CurrentBestY", watch::current_y_best);
 
-    py::class_<watch::TransformedYBest, logger::Property, std::shared_ptr<watch::TransformedYBest>>(t,
-                                                                                                    "TransformedYBest")
-        .def(py::init<std::string, std::string>())
-        .def(py::pickle([](const watch::TransformedYBest &t) { return py::make_tuple(t.name(), t.format()); },
-                        [](py::tuple t) {
-                            return watch::TransformedYBest{t[0].cast<std::string>(), t[1].cast<std::string>()};
-                        }));
-    t.attr("TRANSFORMED_Y_BEST") = py::cast(watch::transformed_y_best);
+    define_property<watch::Violation>(t, "Violation", watch::violation);
+    define_property<watch::Penalty>(t, "Penalty", watch::penalty);
 }
 
 void define_bases(py::module &m)
 {
-    py::class_<Logger, std::shared_ptr<Logger>>(m, "Logger")
-        .def("add_trigger", &Logger::trigger)
-        .def("log", &Logger::log)
-        .def("attach_problem", &Logger::attach_problem)
-        .def("attach_suite", &Logger::attach_suite)
-        .def("call", &Logger::call)
-        .def("reset", &Logger::reset)
-        .def_property_readonly("problem", &Logger::problem);
+    py::class_<Logger, std::shared_ptr<Logger>>(m, "Logger", "Base class for all loggers")
+        .def("add_trigger", &Logger::trigger, "Add a trigger to the logger")
+        .def("call", &Logger::call, "Performs logging behaviour")
+        .def("reset", &Logger::reset, "Reset the state of the logger")
+        .def_property_readonly("problem", &Logger::problem, "Reference to the currently attached problem");
 
     using namespace logger;
-    py::class_<Watcher, Logger, std::shared_ptr<Watcher>>(m, "AbstractWatcher").def("watch", &Watcher::watch);
+    py::class_<Watcher, AbstractWatcher, Logger, std::shared_ptr<Watcher>>(m, "AbstractLogger",
+                                                          "Base class for loggers which track properties")
+        .def(py::init<Triggers, Properties>(), py::arg("triggers") = Triggers{}, py::arg("properties") = Properties{})
+        .def("watch", &Watcher::watch);
 }
 
-
 void define_flatfile(py::module &m)
 {
     using namespace logger;
@@ -299,7 +322,35 @@ void define_flatfile(py::module &m)
              py::arg("triggers"), py::arg("properties"), py::arg("filename") = "IOH.dat",
              py::arg("output_directory") = "./", py::arg("separator") = "\t", py::arg("comment") = "#",
              py::arg("no_value") = "None", py::arg("end_of_line") = "\n", py::arg("repeat_header") = false,
-             py::arg("store_positions") = false, py::arg("common_header_titles") = common_headers)
+             py::arg("store_positions") = false, py::arg("common_header_titles") = common_headers,
+             R"pbdoc(
+                A logger which stores all tracked properties to a file.
+
+                Parameters
+                ----------
+                triggers: list[Trigger]
+                    List of triggers, i.e. when to trigger logging
+                properties: list[Property]
+                    The list of properties to keep track of
+                filename: str = "IOH.dat"
+                    The file to log to
+                output_directory: str = "./"
+                    The path to which to write the file
+                separator: str = "\t"
+                    The column seperator
+                comment: str = "#"
+                    The comment character
+                no_value: str = "None"
+                    The NaN character
+                end_of_line: str = "\n"
+                    The eol character
+                repeat_header: bool = false
+                    Boolean value which indicates whether to repeat the header for every new run.
+                store_positions: bool = false
+                    Boolean value which indicates whether to store the x-positions in the file
+            )pbdoc"
+
+             )
         .def_property_readonly("filename", &FlatFile::filename)
         .def_property_readonly(
             "output_directory",
@@ -319,32 +370,69 @@ void define_store(py::module &m)
 
     using PyStore = PyWatcher<Store>;
     py::class_<PyStore, Watcher, std::shared_ptr<PyStore>>(m, "Store")
-        .def(py::init<Triggers, Properties>())
-        .def("data", py::overload_cast<>(&PyStore::data))
-        .def("at",
-             [](PyStore &f, std::string suite_name, int pb, int dim, int inst, size_t run, size_t evaluation) {
-                 const auto cursor = Store::Cursor(suite_name, pb, dim, inst, run, evaluation);
-                 return f.data(cursor);
-             })
-        .def("watch", py::overload_cast<Property &>(&PyStore::watch))
-        .def("watch", py::overload_cast<const py::object &, const std::string &>(&PyStore::watch))
-        .def("watch", py::overload_cast<const py::object &, const std::vector<std::string> &>(&PyStore::watch))
+        .def(py::init<Triggers, Properties>(), py::arg("triggers"), py::arg("properties"),
+             R"pbdoc(
+                A logger which stores all tracked properties in memory.
+
+                Parameters
+                ----------
+                triggers: list[Trigger]
+                    List of triggers, i.e. when to trigger logging
+                properties: list[Property]
+                    The list of properties to keep track of
+            )pbdoc"
+
+             )
+        .def("data", py::overload_cast<>(&PyStore::data), "Accessor to the internal data container")
+        .def(
+            "at",
+            [](PyStore &f, std::string suite_name, int pb, int dim, int inst, size_t run, size_t evaluation) {
+                const auto cursor = Store::Cursor(suite_name, pb, dim, inst, run, evaluation);
+                return f.data(cursor);
+            },
+            "Accessor for a specific stored record")
+        .def("watch", py::overload_cast<Property &>(&PyStore::watch), "Add a property")
+        .def("watch", py::overload_cast<const py::object &, const std::string &>(&PyStore::watch), "Add a property")
+        .def("watch", py::overload_cast<const py::object &, const std::vector<std::string> &>(&PyStore::watch),
+             "Add multliple properties")
         .def("__repr__", [](PyStore &f) {
             return fmt::format("<Store (suites: ({}),)>", fmt::join(ioh::common::keys(f.data()), ","));
         });
 }
 
+template<typename A>
 void define_analyzer(py::module &m)
 {
     using namespace logger;
     Triggers def_trigs{trigger::on_improvement};
     Properties def_props{};
+    using PyAnalyzer = PyAnalyzer<A>;
     py::class_<PyAnalyzer, Watcher, std::shared_ptr<PyAnalyzer>>(m, "Analyzer")
         .def(py::init<Triggers, Properties, fs::path, std::string, std::string, std::string, bool>(),
              py::arg("triggers") = def_trigs, py::arg("additional_properties") = def_props,
              py::arg("root") = fs::current_path(), py::arg("folder_name") = "ioh_data",
              py::arg("algorithm_name") = "algorithm_name", py::arg("algorithm_info") = "algorithm_info",
-             py::arg("store_positions") = false)
+             py::arg("store_positions") = false,
+             R"pbdoc(
+                A logger which stores all tracked properties to a file.
+
+                Parameters
+                ----------
+                triggers: list[Trigger]
+                    List of triggers, i.e. when to trigger logging
+                additional_properties: list[Property]
+                    The list of additional properties to keep track of (additional to the default properties for this logger type)
+                root: str = "./"
+                    The path to which to write the files
+                folder_name: str = "./"
+                    The name of the folder to which to write the files
+                algorithm_name: str = "algorithm_name"
+                    Optional name parameter for the algorithm to be added to the info files
+                algorithm_info: str = "algorithm_info"
+                    Optional info parameter for the algorithm to be added to the info files
+                store_positions: bool = false
+                    Boolean value which indicates whether to store the x-positions in the file
+            )pbdoc")
         .def("add_experiment_attribute", &PyAnalyzer::add_experiment_attribute)
         .def("set_experiment_attributes", &PyAnalyzer::set_experiment_attributes)
         .def("add_run_attributes",
@@ -357,6 +445,7 @@ void define_analyzer(py::module &m)
         .def("set_run_attributes", &PyAnalyzer::set_run_attributes_python)
         .def("set_run_attribute", &PyAnalyzer::set_run_attribute_python)
         .def_property_readonly("output_directory", &PyAnalyzer::output_directory)
+        .def("close", &PyAnalyzer::close)
         .def("watch", py::overload_cast<Property &>(&PyAnalyzer::watch))
         .def("watch", py::overload_cast<const py::object &, const std::string &>(&PyAnalyzer::watch))
         .def("watch", py::overload_cast<const py::object &, const std::vector<std::string> &>(&PyAnalyzer::watch))
@@ -396,7 +485,7 @@ void define_eah(py::module &m)
     define_eah_scale<double>(eah, "RealScale");
     define_eah_scale<size_t>(eah, "IntegerScale");
 
-    py::class_<EAH, Logger, std::shared_ptr<EAH>>(m, "EAH")
+    py::class_<EAH, Logger, std::shared_ptr<EAH>>(m, "EAH", "Emperical Attainment Histogram Logger")
         .def(py::init<double, double, size_t, size_t, size_t, size_t>(), py::arg("error_min"), py::arg("error_max"),
              py::arg("error_buckets"), py::arg("evals_min"), py::arg("evals_max"), py::arg("evals_buckets"))
         .def(py::init<eah::LinearScale<double> &, eah::LinearScale<size_t> &>(), py::arg("error_scale"),
@@ -427,7 +516,7 @@ void define_eaf(py::module &m)
         .def("__repr__",
              [](const eaf::RunPoint &p) { return fmt::format("<RunPoint {} {} {}>", p.qual, p.time, p.run); });
 
-    py::class_<EAF, Logger, std::shared_ptr<EAF>>(m, "EAF")
+    py::class_<EAF, Logger, std::shared_ptr<EAF>>(m, "EAF", "Emperical Attainment Function Logger")
         .def(py::init<>())
         .def_property_readonly("data", py::overload_cast<>(&EAF::data, py::const_))
         .def("at", [](EAF &f, std::string suite_name, int pb, int dim, int inst, size_t run) {
@@ -436,23 +525,24 @@ void define_eaf(py::module &m)
         });
 }
 
-
 void define_loggers(py::module &m)
 {
     using namespace logger;
-    py::class_<Combine, Logger, std::shared_ptr<Combine>>(m, "Combine")
+    py::class_<Combine, Logger, std::shared_ptr<Combine>>(m, "Combine", "Utility class to combine multiple loggers")
         .def(py::init<std::vector<std::reference_wrapper<Logger>>>(), py::arg("loggers"))
         .def(py::init<std::reference_wrapper<Logger>>(), py::arg("logger"))
         .def("append", &Combine::append);
 
     define_flatfile(m);
     define_store(m);
-    define_analyzer(m);
+    auto old = m.def_submodule("old");
+    define_analyzer<logger::analyzer::v1::Analyzer>(old);
+    define_analyzer<logger::analyzer::v2::Analyzer>(m);
+
     define_eah(m);
     define_eaf(m);
 }
 
-
 void define_logger(py::module &m)
 {
     py::class_<fs::path>(m, "Path").def(py::init<std::string>());
