@@ -1,21 +1,11 @@
 import os
-import random
 import unittest
-import shutil
 import math
 
 import ioh
 
-class Algorithm:
-    def __init__(self):
-        self.x = 10
-        self.i = 1
-    
-    def __call__(self, p: ioh.problem.Real):
-        for i in range(10000):
-            x = list(map(lambda x: random.random(), range(p.meta_data.n_variables)))    
-            p(x)
-            self.i = i
+DATA_DIR = os.path.join(
+    os.path.dirname(os.path.dirname(os.path.dirname(__file__))), "static")
 
 class wmodel(ioh.problem.AbstractWModel):
     def __init__(self, instance, dim):
@@ -25,10 +15,6 @@ class wmodel(ioh.problem.AbstractWModel):
         return x.count(1)
 
 
-
-DATA_DIR = os.path.join(
-    os.path.dirname(os.path.dirname(__file__)), "static")
-
 class TestProblem(unittest.TestCase):
     def test_get_problem(self):
         self.assertIsInstance(ioh.get_problem(1, 1, 2, "BBOB"), ioh.problem.Sphere)
@@ -41,50 +27,21 @@ class TestProblem(unittest.TestCase):
         for p in map(lambda x: x(1, 10), (wmodel, ioh.problem.WModelLeadingOnes, ioh.problem.WModelOneMax)):
             self.assertEqual(p([1] * 10), 10)
 
-    def test_experimenter(self):
-        exp = ioh.Experiment(
-            Algorithm(),
-            [1], [1, 2], [5],
-            njobs = 1,
-            reps = 2,
-            experiment_attributes = {"a": "1"},
-            run_attributes = ['x'],
-            logged_attributes = ['i']
-        )
+    def test_enforced_bounds(self):
+        p = ioh.get_problem(1, 1, 2)
+        x0 = [10, 1]
+        y0 = p(x0)
+        p.enforce_bounds()
+        y1 = p(x0)
+        self.assertEqual(y0 + p.constraints.penalty(), y1)
 
-        def a_problem(x):
-            return 0.0
-            
-        exp.add_custom_problem(a_problem, "Name")
-        exp()
+    def test_custom_constraint(self):
+        p = ioh.get_problem(1, 1, 2)
+        c = ioh.RealConstraint(lambda x: float(x[0] > 1), 25.0)
+        y0 = p([10, 1])
+        p.add_constraint(c)
+        self.assertEqual(p([10, 1]), y0 + 25)
 
-        info_files = {'IOHprofiler_f25_Name.info', 'IOHprofiler_f1_Sphere.info'}
-        data_files = {'IOHprofiler_f25_DIM5.dat', 'IOHprofiler_f1_DIM5.dat'}
-
-        for item in os.listdir('ioh_data'):
-            path = os.path.join('ioh_data', item)
-            if os.path.isfile(path) and item in info_files:
-                self.assertNotEqual(os.path.getsize(path), 0)
-                info_files.remove(item)
-            elif os.path.isdir(path):
-                for f in os.listdir(path):
-                    if f in data_files:
-                        path = os.path.join(path, f)
-                        self.assertNotEqual(os.path.getsize(path), 0)
-                        with open(path) as h:
-                            data = list(filter(lambda x:x.startswith('"'), h))       
-
-                        self.assertEqual(len(data), 4)
-                        data_files.remove(f)
-
-        self.assertSetEqual(info_files, set())
-        self.assertSetEqual(data_files, set())
-
-        self.assertTrue(os.path.isfile("ioh_data.zip"))
-
-        shutil.rmtree("ioh_data")
-        os.remove("ioh_data.zip")
-           
     def test_evaluation_bbob_problems(self):
         for fid in range(1,25):
             f = ioh.get_problem(fid, 1 ,5, "BBOB")
@@ -95,6 +52,12 @@ class TestProblem(unittest.TestCase):
             f = ioh.get_problem(fid, 1 ,4, "PBO")
             self.assertGreater(f([0,0,0,0]), -1000) 
 
+    def test_has_submodular(self):
+        problems = list(getattr(ioh.problem, "GraphProblem").problems.values())
+        self.assertNotEqual(0, len(problems))
+        for name in ("MaxCut", "MaxInfluence", "MaxCoverage", "PackWhileTravel"):
+            self.assertNotEqual(0, len([p for p in problems if name in p]))
+
     def test_bbob_problems_first_instance(self):
         expected = [
             161.17445568,
@@ -122,7 +85,7 @@ class TestProblem(unittest.TestCase):
             18.635078550302751,
             1782.2733296400438,
         ]
-        for i in sorted(ioh.problem.BBOB.problems.keys()):
+        for i in sorted(ioh.problem.BBOB.problems.keys())[:24]:
             p = ioh.problem.BBOB.create(i, 1, 5)
             self.assertTrue(
                 math.isclose(p([0.1, 1., 2., 4., 5.4]), expected[i-1])
@@ -156,7 +119,7 @@ class TestProblem(unittest.TestCase):
             0.45,
             -0.70717,
         ]
-        for i in sorted(ioh.problem.PBO.problems.keys()):
+        for i in sorted(ioh.problem.PBO.problems.keys())[:24]:
             p = ioh.problem.PBO.create(i, 1, 9)
             y = p([1, 1, 0, 1, 0, 0, 0, 1, 1])
             self.assertTrue(math.isclose(y, expected[i-1], abs_tol = 0.000099),
@@ -182,32 +145,6 @@ class TestProblem(unittest.TestCase):
                         self.assertTrue(math.isclose(p(x), float(y), abs_tol = tol))
 
 
-    def test_wrap_problem_scoped(self):
-        def w():
-            l = lambda x: 0.0
-            p = ioh.problem.wrap_real_problem(l, "l")
-            return p
-        p = w()
-        y = p([0]*5)
-        self.assertEqual(y, 0.0)
-
-    def test_wrap_problem(self):
-        l = lambda x: 0.0
-        p = ioh.problem.wrap_real_problem(l, "f")
-        p([0]*5)
-
-        p2 = ioh.problem.wrap_real_problem(l, "f")
-        self.assertEqual(p.meta_data.problem_id, p2.meta_data.problem_id)
-        self.assertEqual(p.meta_data.name, p2.meta_data.name)
-
-    def test_wrap_problem_builtins(self):
-        for f in (sum, min, max):
-            p = ioh.problem.wrap_real_problem(f, "f")
-            y = p([0]*5)
-            self.assertEqual(y, 0.0)
-
-
-
 
 if __name__ == "__main__":
     unittest.main()
